@@ -18,8 +18,8 @@ use asyncgit::{
 		extract_username_password, need_username_password,
 		BasicAuthCredential,
 	},
-	sync::{get_tags_with_metadata, TagWithMetadata},
-	AsyncGitNotification, CWD,
+	sync::{get_tags_with_metadata, RepoPathRef, TagWithMetadata},
+	AsyncGitNotification,
 };
 use crossbeam_channel::Sender;
 use crossterm::event::Event;
@@ -38,6 +38,7 @@ use ui::style::SharedTheme;
 
 ///
 pub struct TagListComponent {
+	repo: RepoPathRef,
 	theme: SharedTheme,
 	queue: Queue,
 	tags: Option<Vec<TagWithMetadata>>,
@@ -177,25 +178,25 @@ impl Component for TagListComponent {
 	fn event(&mut self, event: Event) -> Result<EventState> {
 		if self.visible {
 			if let Event::Key(key) = event {
-				if key == self.key_config.exit_popup {
+				if key == self.key_config.keys.exit_popup {
 					self.hide();
-				} else if key == self.key_config.move_up {
+				} else if key == self.key_config.keys.move_up {
 					self.move_selection(ScrollType::Up);
-				} else if key == self.key_config.move_down {
+				} else if key == self.key_config.keys.move_down {
 					self.move_selection(ScrollType::Down);
-				} else if key == self.key_config.shift_up
-					|| key == self.key_config.home
+				} else if key == self.key_config.keys.shift_up
+					|| key == self.key_config.keys.home
 				{
 					self.move_selection(ScrollType::Home);
-				} else if key == self.key_config.shift_down
-					|| key == self.key_config.end
+				} else if key == self.key_config.keys.shift_down
+					|| key == self.key_config.keys.end
 				{
 					self.move_selection(ScrollType::End);
-				} else if key == self.key_config.page_down {
+				} else if key == self.key_config.keys.page_down {
 					self.move_selection(ScrollType::PageDown);
-				} else if key == self.key_config.page_up {
+				} else if key == self.key_config.keys.page_up {
 					self.move_selection(ScrollType::PageUp);
-				} else if key == self.key_config.delete_tag {
+				} else if key == self.key_config.keys.delete_tag {
 					return self.selected_tag().map_or(
 						Ok(EventState::NotConsumed),
 						|tag| {
@@ -209,7 +210,7 @@ impl Component for TagListComponent {
 							Ok(EventState::Consumed)
 						},
 					);
-				} else if key == self.key_config.select_tag {
+				} else if key == self.key_config.keys.select_tag {
 					return self.selected_tag().map_or(
 						Ok(EventState::NotConsumed),
 						|tag| {
@@ -221,7 +222,7 @@ impl Component for TagListComponent {
 							Ok(EventState::Consumed)
 						},
 					);
-				} else if key == self.key_config.push {
+				} else if key == self.key_config.keys.push {
 					self.queue.push(InternalEvent::PushTags);
 				}
 			}
@@ -249,6 +250,7 @@ impl Component for TagListComponent {
 
 impl TagListComponent {
 	pub fn new(
+		repo: RepoPathRef,
 		queue: &Queue,
 		sender: &Sender<AsyncGitNotification>,
 		theme: SharedTheme,
@@ -265,6 +267,7 @@ impl TagListComponent {
 			missing_remote_tags: None,
 			async_remote_tags: AsyncSingleJob::new(sender.clone()),
 			key_config,
+			repo,
 		}
 	}
 
@@ -273,17 +276,19 @@ impl TagListComponent {
 		self.table_state.get_mut().select(Some(0));
 		self.show()?;
 
-		let basic_credential = if need_username_password()? {
-			let credential = extract_username_password()?;
+		let basic_credential =
+			if need_username_password(&self.repo.borrow())? {
+				let credential =
+					extract_username_password(&self.repo.borrow())?;
 
-			if credential.is_complete() {
-				Some(credential)
+				if credential.is_complete() {
+					Some(credential)
+				} else {
+					None
+				}
 			} else {
 				None
-			}
-		} else {
-			None
-		};
+			};
 
 		self.basic_credential = basic_credential;
 
@@ -320,7 +325,7 @@ impl TagListComponent {
 
 	/// fetch list of tags
 	pub fn update_tags(&mut self) -> Result<()> {
-		let tags = get_tags_with_metadata(CWD)?;
+		let tags = get_tags_with_metadata(&self.repo.borrow())?;
 
 		self.tags = Some(tags);
 
@@ -329,6 +334,7 @@ impl TagListComponent {
 
 	pub fn update_missing_remote_tags(&mut self) {
 		self.async_remote_tags.spawn(AsyncRemoteTagsJob::new(
+			self.repo.borrow().clone(),
 			self.basic_credential.clone(),
 		));
 	}
@@ -368,11 +374,9 @@ impl TagListComponent {
 
 	///
 	fn get_rows(&self) -> Vec<Row> {
-		if let Some(ref tags) = self.tags {
+		self.tags.as_ref().map_or_else(Vec::new, |tags| {
 			tags.iter().map(|tag| self.get_row(tag)).collect()
-		} else {
-			vec![]
-		}
+		})
 	}
 
 	///
