@@ -1,7 +1,11 @@
 use crate::{components::AppOption, tabs::StashingOptions};
-use asyncgit::sync::{diff::DiffLinePosition, CommitId, CommitTags};
+use asyncgit::sync::{
+	diff::DiffLinePosition, CommitId, CommitTags, TreeFile,
+};
 use bitflags::bitflags;
-use std::{cell::RefCell, collections::VecDeque, rc::Rc};
+use std::{
+	cell::RefCell, collections::VecDeque, path::PathBuf, rc::Rc,
+};
 
 bitflags! {
 	/// flags defining what part of the app need to update
@@ -32,11 +36,13 @@ pub enum Action {
 	ResetLines(String, Vec<DiffLinePosition>),
 	StashDrop(Vec<CommitId>),
 	StashPop(CommitId),
-	DeleteBranch(String, bool),
+	DeleteLocalBranch(String),
+	DeleteRemoteBranch(String),
 	DeleteTag(String),
 	ForcePush(String, bool),
 	PullMerge { incoming: usize, rebase: bool },
 	AbortMerge,
+	AbortRebase,
 }
 
 ///
@@ -87,6 +93,12 @@ pub enum InternalEvent {
 	OpenFileTree(CommitId),
 	///
 	OptionSwitched(AppOption),
+	///
+	OpenFileFinder(Vec<TreeFile>),
+	///
+	FileFinderChanged(Option<PathBuf>),
+	///
+	FetchRemotes,
 }
 
 /// single threaded simple queue for components to communicate with each other
