@@ -14,20 +14,21 @@ use crate::{
 use anyhow::Result;
 use asyncgit::{
 	cached,
-	sync::BranchCompare,
-	sync::{self, status::StatusType, RepoState},
+	sync::{
+		self, status::StatusType, RepoPath, RepoPathRef, RepoState,
+	},
+	sync::{BranchCompare, CommitId},
 	AsyncDiff, AsyncGitNotification, AsyncStatus, DiffParams,
-	DiffType, StatusParams, CWD,
+	DiffType, StatusParams,
 };
 use crossbeam_channel::Sender;
 use crossterm::event::Event;
 use itertools::Itertools;
 use std::convert::Into;
-use std::convert::TryFrom;
 use tui::{
 	layout::{Alignment, Constraint, Direction, Layout},
 	style::{Color, Style},
-	widgets::Paragraph,
+	widgets::{Block, BorderType, Borders, Paragraph},
 };
 
 /// what part of the screen is focused
@@ -57,6 +58,7 @@ enum DiffTarget {
 }
 
 pub struct Status {
+	repo: RepoPathRef,
 	visible: bool,
 	focus: Focus,
 	diff_target: DiffTarget,
@@ -80,6 +82,19 @@ impl DrawableComponent for Status {
 		f: &mut tui::Frame<B>,
 		rect: tui::layout::Rect,
 	) -> Result<()> {
+		let repo_unclean = self.repo_state_unclean();
+		let rects = if repo_unclean {
+			Layout::default()
+				.direction(Direction::Vertical)
+				.constraints(
+					[Constraint::Min(1), Constraint::Length(3)]
+						.as_ref(),
+				)
+				.split(rect)
+		} else {
+			vec![rect]
+		};
+
 		let chunks = Layout::default()
 			.direction(Direction::Horizontal)
 			.constraints(
@@ -96,7 +111,7 @@ impl DrawableComponent for Status {
 				}
 				.as_ref(),
 			)
-			.split(rect);
+			.split(rects[0]);
 
 		let left_chunks = Layout::default()
 			.direction(Direction::Vertical)
@@ -120,7 +135,10 @@ impl DrawableComponent for Status {
 		self.index.draw(f, left_chunks[1])?;
 		self.diff.draw(f, chunks[1])?;
 		self.draw_branch_state(f, &left_chunks);
-		Self::draw_repo_state(f, left_chunks[0])?;
+
+		if repo_unclean {
+			self.draw_repo_state(f, rects[1]);
+		}
 
 		Ok(())
 	}
@@ -131,47 +149,61 @@ impl Status {
 
 	///
 	pub fn new(
+		repo: RepoPathRef,
 		queue: &Queue,
 		sender: &Sender<AsyncGitNotification>,
 		theme: SharedTheme,
 		key_config: SharedKeyConfig,
 		options: SharedOptions,
 	) -> Self {
+		let repo_clone = repo.borrow().clone();
 		Self {
 			queue: queue.clone(),
 			visible: true,
 			focus: Focus::WorkDir,
 			diff_target: DiffTarget::WorkingDir,
 			index_wd: ChangesComponent::new(
+				repo.clone(),
 				&strings::title_status(&key_config),
 				true,
 				true,
 				queue.clone(),
 				theme.clone(),
 				key_config.clone(),
+				options.clone(),
 			),
 			index: ChangesComponent::new(
+				repo.clone(),
 				&strings::title_index(&key_config),
 				false,
 				false,
 				queue.clone(),
 				theme.clone(),
 				key_config.clone(),
+				options.clone(),
 			),
 			diff: DiffComponent::new(
+				repo.clone(),
 				queue.clone(),
 				theme,
 				key_config.clone(),
 				false,
 			),
-			git_diff: AsyncDiff::new(sender),
-			git_status_workdir: AsyncStatus::new(sender.clone()),
-			git_status_stage: AsyncStatus::new(sender.clone()),
+			git_diff: AsyncDiff::new(repo_clone.clone(), sender),
+			git_status_workdir: AsyncStatus::new(
+				repo_clone.clone(),
+				sender.clone(),
+			),
+			git_status_stage: AsyncStatus::new(
+				repo_clone,
+				sender.clone(),
+			),
 			git_action_executed: false,
 			git_branch_state: None,
-			git_branch_name: cached::BranchName::new(CWD),
+			git_branch_name: cached::BranchName::new(repo.clone()),
 			key_config,
 			options,
+			repo,
 		}
 	}
 
@@ -181,15 +213,16 @@ impl Status {
 		chunks: &[tui::layout::Rect],
 	) {
 		if let Some(branch_name) = self.git_branch_name.last() {
-			let ahead_behind =
-				if let Some(state) = &self.git_branch_state {
+			let ahead_behind = self
+				.git_branch_state
+				.as_ref()
+				.map_or_else(String::new, |state| {
 					format!(
 						"\u{2191}{} \u{2193}{} ",
 						state.ahead, state.behind,
 					)
-				} else {
-					String::new()
-				};
+				});
+
 			let w = Paragraph::new(format!(
 				"{}{{{}}}",
 				ahead_behind, branch_name
@@ -214,42 +247,75 @@ impl Status {
 		}
 	}
 
+	fn repo_state_text(repo: &RepoPath, state: &RepoState) -> String {
+		match state {
+			RepoState::Merge => {
+				let ids =
+					sync::mergehead_ids(repo).unwrap_or_default();
+
+				format!(
+					"Commits: {}",
+					ids.iter()
+						.map(sync::CommitId::get_short_string)
+						.join(",")
+				)
+			}
+			RepoState::Rebase => {
+				if let Ok(p) = sync::rebase_progress(repo) {
+					format!(
+						"Step: {}/{} Current Commit: {}",
+						p.current + 1,
+						p.steps,
+						p.current_commit
+							.as_ref()
+							.map(CommitId::get_short_string)
+							.unwrap_or_default(),
+					)
+				} else {
+					String::new()
+				}
+			}
+			_ => format!("{:?}", state),
+		}
+	}
+
 	fn draw_repo_state<B: tui::backend::Backend>(
+		&self,
 		f: &mut tui::Frame<B>,
 		r: tui::layout::Rect,
-	) -> Result<()> {
-		if let Ok(state) = sync::repo_state(CWD) {
+	) {
+		if let Ok(state) = sync::repo_state(&self.repo.borrow()) {
 			if state != RepoState::Clean {
-				let ids =
-					sync::mergehead_ids(CWD).unwrap_or_default();
-				let ids = format!(
-					"({})",
-					ids.iter()
-						.map(|id| sync::CommitId::get_short_string(
-							id
-						))
-						.join(",")
+				let txt = Self::repo_state_text(
+					&self.repo.borrow(),
+					&state,
 				);
-				let txt = format!("{:?} {}", state, ids);
-				let txt_len = u16::try_from(txt.len())?;
+
 				let w = Paragraph::new(txt)
+					.block(
+						Block::default()
+							.border_type(BorderType::Plain)
+							.borders(Borders::all())
+							.border_style(
+								Style::default().fg(Color::Yellow),
+							)
+							.title(format!("Pending {:?}", state)),
+					)
 					.style(Style::default().fg(Color::Red))
 					.alignment(Alignment::Left);
 
-				let mut rect = r;
-				rect.x += 1;
-				rect.width =
-					rect.width.saturating_sub(2).min(txt_len);
-				rect.y += rect.height.saturating_sub(1);
-				rect.height = rect
-					.height
-					.saturating_sub(rect.height.saturating_sub(1));
-
-				f.render_widget(w, rect);
+				f.render_widget(w, r);
 			}
 		}
+	}
 
-		Ok(())
+	fn repo_state_unclean(&self) -> bool {
+		if let Ok(state) = sync::repo_state(&self.repo.borrow()) {
+			if state != RepoState::Clean {
+				return true;
+			}
+		}
+		false
 	}
 
 	fn can_focus_diff(&self) -> bool {
@@ -354,7 +420,7 @@ impl Status {
 			AsyncGitNotification::Diff => self.update_diff()?,
 			AsyncGitNotification::Status => self.update_status()?,
 			AsyncGitNotification::Push
-			| AsyncGitNotification::Fetch
+			| AsyncGitNotification::Pull
 			| AsyncGitNotification::CommitFiles => {
 				self.branch_compare();
 			}
@@ -450,7 +516,10 @@ impl Status {
 
 	/// called after confirmation
 	pub fn reset(&mut self, item: &ResetItem) -> bool {
-		if let Err(e) = sync::reset_workdir(CWD, item.path.as_str()) {
+		if let Err(e) = sync::reset_workdir(
+			&self.repo.borrow(),
+			item.path.as_str(),
+		) {
 			self.queue.push(InternalEvent::ShowErrorMsg(format!(
 				"reset failed:\n{}",
 				e
@@ -495,15 +564,18 @@ impl Status {
 		try_or_popup!(
 			self,
 			"undo commit failed:",
-			sync::utils::undo_last_commit(CWD)
+			sync::utils::undo_last_commit(&self.repo.borrow())
 		);
 	}
 
 	fn branch_compare(&mut self) {
 		self.git_branch_state =
 			self.git_branch_name.last().and_then(|branch| {
-				sync::branch_compare_upstream(CWD, branch.as_str())
-					.ok()
+				sync::branch_compare_upstream(
+					&self.repo.borrow(),
+					branch.as_str(),
+				)
+				.ok()
 			});
 	}
 
@@ -513,13 +585,40 @@ impl Status {
 			.map_or(true, |state| state.ahead > 0)
 	}
 
-	fn can_abort_merge() -> bool {
-		sync::repo_state(CWD).unwrap_or(RepoState::Clean)
+	fn can_abort_merge(&self) -> bool {
+		sync::repo_state(&self.repo.borrow())
+			.unwrap_or(RepoState::Clean)
 			== RepoState::Merge
 	}
 
+	fn pending_rebase(&self) -> bool {
+		sync::repo_state(&self.repo.borrow())
+			.unwrap_or(RepoState::Clean)
+			== RepoState::Rebase
+	}
+
 	pub fn abort_merge(&self) {
-		try_or_popup!(self, "abort merge", sync::abort_merge(CWD));
+		try_or_popup!(
+			self,
+			"abort merge",
+			sync::abort_merge(&self.repo.borrow())
+		);
+	}
+
+	pub fn abort_rebase(&self) {
+		try_or_popup!(
+			self,
+			"abort rebase",
+			sync::abort_pending_rebase(&self.repo.borrow())
+		);
+	}
+
+	fn continue_rebase(&self) {
+		try_or_popup!(
+			self,
+			"continue rebase",
+			sync::continue_pending_rebase(&self.repo.borrow())
+		);
 	}
 
 	fn commands_nav(
@@ -565,6 +664,12 @@ impl Status {
 			.order(strings::order::NAV),
 		);
 	}
+
+	fn can_commit(&self) -> bool {
+		self.index.focused()
+			&& !self.index.is_empty()
+			&& !self.pending_rebase()
+	}
 }
 
 impl Component for Status {
@@ -582,6 +687,15 @@ impl Component for Status {
 				self.components().as_slice(),
 			);
 
+			out.push(
+				CommandInfo::new(
+					strings::commands::commit_open(&self.key_config),
+					true,
+					self.can_commit() || force_all,
+				)
+				.order(-1),
+			);
+
 			out.push(CommandInfo::new(
 				strings::commands::open_branch_select_popup(
 					&self.key_config,
@@ -611,13 +725,25 @@ impl Component for Status {
 			out.push(CommandInfo::new(
 				strings::commands::undo_commit(&self.key_config),
 				true,
-				!focus_on_diff,
+				(!self.pending_rebase() && !focus_on_diff)
+					|| force_all,
 			));
 
 			out.push(CommandInfo::new(
 				strings::commands::abort_merge(&self.key_config),
 				true,
-				Self::can_abort_merge() || force_all,
+				self.can_abort_merge() || force_all,
+			));
+
+			out.push(CommandInfo::new(
+				strings::commands::continue_rebase(&self.key_config),
+				true,
+				self.pending_rebase() || force_all,
+			));
+			out.push(CommandInfo::new(
+				strings::commands::abort_rebase(&self.key_config),
+				true,
+				self.pending_rebase() || force_all,
 			));
 		}
 
@@ -638,6 +764,7 @@ impl Component for Status {
 		visibility_blocking(self)
 	}
 
+	#[allow(clippy::too_many_lines, clippy::cognitive_complexity)]
 	fn event(
 		&mut self,
 		ev: crossterm::event::Event,
@@ -651,7 +778,7 @@ impl Component for Status {
 			}
 
 			if let Event::Key(k) = ev {
-				return if k == self.key_config.edit_file
+				return if k == self.key_config.keys.edit_file
 					&& (self.can_focus_diff()
 						|| self.is_focus_on_diff())
 				{
@@ -663,53 +790,58 @@ impl Component for Status {
 						);
 					}
 					Ok(EventState::Consumed)
-				} else if k == self.key_config.toggle_workarea
+				} else if k == self.key_config.keys.open_commit
+					&& self.can_commit()
+				{
+					self.queue.push(InternalEvent::OpenCommit);
+					Ok(EventState::Consumed)
+				} else if k == self.key_config.keys.toggle_workarea
 					&& !self.is_focus_on_diff()
 				{
 					self.switch_focus(self.focus.toggled_focus())
 						.map(Into::into)
-				} else if k == self.key_config.focus_right
+				} else if k == self.key_config.keys.focus_right
 					&& self.can_focus_diff()
 				{
 					self.switch_focus(Focus::Diff).map(Into::into)
-				} else if k == self.key_config.focus_left {
+				} else if k == self.key_config.keys.focus_left {
 					self.switch_focus(match self.diff_target {
 						DiffTarget::Stage => Focus::Stage,
 						DiffTarget::WorkingDir => Focus::WorkDir,
 					})
 					.map(Into::into)
-				} else if k == self.key_config.move_down
+				} else if k == self.key_config.keys.move_down
 					&& self.focus == Focus::WorkDir
 					&& !self.index.is_empty()
 				{
 					self.switch_focus(Focus::Stage).map(Into::into)
-				} else if k == self.key_config.move_up
+				} else if k == self.key_config.keys.move_up
 					&& self.focus == Focus::Stage
 					&& !self.index_wd.is_empty()
 				{
 					self.switch_focus(Focus::WorkDir).map(Into::into)
-				} else if k == self.key_config.select_branch
+				} else if k == self.key_config.keys.select_branch
 					&& !self.is_focus_on_diff()
 				{
 					self.queue.push(InternalEvent::SelectBranch);
 					Ok(EventState::Consumed)
-				} else if k == self.key_config.force_push
+				} else if k == self.key_config.keys.force_push
 					&& !self.is_focus_on_diff()
 					&& self.can_push()
 				{
 					self.push(true);
 					Ok(EventState::Consumed)
-				} else if k == self.key_config.push
+				} else if k == self.key_config.keys.push
 					&& !self.is_focus_on_diff()
 				{
 					self.push(false);
 					Ok(EventState::Consumed)
-				} else if k == self.key_config.pull
+				} else if k == self.key_config.keys.pull
 					&& !self.is_focus_on_diff()
 				{
 					self.pull();
 					Ok(EventState::Consumed)
-				} else if k == self.key_config.undo_commit
+				} else if k == self.key_config.keys.undo_commit
 					&& !self.is_focus_on_diff()
 				{
 					self.undo_last_commit();
@@ -717,13 +849,29 @@ impl Component for Status {
 						NeedsUpdate::ALL,
 					));
 					Ok(EventState::Consumed)
-				} else if k == self.key_config.abort_merge
-					&& Self::can_abort_merge()
+				} else if k == self.key_config.keys.abort_merge
+					&& self.can_abort_merge()
 				{
 					self.queue.push(InternalEvent::ConfirmAction(
 						Action::AbortMerge,
 					));
 
+					Ok(EventState::Consumed)
+				} else if k == self.key_config.keys.abort_merge
+					&& self.pending_rebase()
+				{
+					self.queue.push(InternalEvent::ConfirmAction(
+						Action::AbortRebase,
+					));
+
+					Ok(EventState::Consumed)
+				} else if k == self.key_config.keys.rebase_branch
+					&& self.pending_rebase()
+				{
+					self.continue_rebase();
+					self.queue.push(InternalEvent::Update(
+						NeedsUpdate::ALL,
+					));
 					Ok(EventState::Consumed)
 				} else {
 					Ok(EventState::NotConsumed)
